; Shifter - serial input module for BLINKENmini, LittleLights and BlinkenLeds
; interrupt code
; version 0.1 date 2006-02-20
; Copyright (C) 2005-2006 Stefan Schuermans <1stein@schuermans.info>
; Copyleft: GNU public license - http://www.gnu.org/copyleft/gpl.html
; a BlinkenArea project - http://www.blinkenarea.org/



; do step actions
;  - for i = 0..6:
;     - in step i^2: output black/white picture, turn on pixel with value > i
;  - in step 42: read SNES keys (1st half)
;  - in step 46: read SNES keys (2nd half)
;  - uses: X, SNES_FLAG, I_REG0, I_REG1, I_REG2, I_CNT, I_TMP
;  - used stack depth (without call to this function): 2
STEP_ACTIONS:
        cpi     STEP,10
        brsh    STEP_ACTIONS_10
        cpi     STEP,3
        brsh    STEP_ACTIONS_3
; in step 0: output black/white picture, turn on pixel with value >= 1
; in step 1: output black/white picture, turn on pixel with value >= 2
        cpi     STEP,0
        brne    STEP_ACTIONS_NOT_0
        rjmp    OUT_BW_1
STEP_ACTIONS_NOT_0:
        cpi     STEP,1
        brne    STEP_ACTIONS_NOT_1
        rjmp    OUT_BW_2
STEP_ACTIONS_NOT_1:
        ret
STEP_ACTIONS_3:
; in step 4: output black/white picture, turn on pixel with value >= 3
; in step 9: output black/white picture, turn on pixel with value >= 4
        cpi     STEP,4
        brne    STEP_ACTIONS_NOT_4
        rjmp    OUT_BW_3
STEP_ACTIONS_NOT_4:
        cpi     STEP,9
        brne    STEP_ACTIONS_NOT_9
        rjmp    OUT_BW_4
STEP_ACTIONS_NOT_9:
        ret
STEP_ACTIONS_10:
        cpi     STEP,30
        brsh    STEP_ACTIONS_30
; in step 16: output black/white picture, turn on pixel with value >= 5
; in step 25: output black/white picture, turn on pixel with value >= 6
        cpi     STEP,16
        brne    STEP_ACTIONS_NOT_16
        rjmp    OUT_BW_5
STEP_ACTIONS_NOT_16:
        cpi     STEP,25
        brne    STEP_ACTIONS_NOT_25
        rjmp    OUT_BW_6
STEP_ACTIONS_NOT_25:
        ret
STEP_ACTIONS_30:
; in step 36: output black/white picture, turn on pixel with value >= 7
; in step 42: read SNES keys (1st half)
; in step 46: read SNES keys (2nd half)
        cpi     STEP,36
        brne    STEP_ACTIONS_NOT_36
        rjmp    OUT_BW_7
STEP_ACTIONS_NOT_36:
.IF CONF_SNES
        cpi     STEP,42
        brne    STEP_ACTIONS_NOT_42
        rjmp    IN_SNES_1
STEP_ACTIONS_NOT_42:
        cpi     STEP,46
        brne    STEP_ACTIONS_NOT_46
        rjmp    IN_SNES_2
STEP_ACTIONS_NOT_46:
.ENDIF
        ret



; timer interrupt
;  - every 192 us (2304 cycles)
; uses: X (reserved for interrupt), I_SREG, I_TMP, I_CNT, I_REG0, I_REG1, I_REG2, STEP
;  - used stack depth (without call to this function): 4
INT_TIMER:
        in      I_SREG,SREG
; do step actions
        rcall   STEP_ACTIONS
; increment output step number
        inc     STEP
        cpi     STEP,49
        brlo    INT_TIMER_STEP_LOWER_49
        clr     STEP
INT_TIMER_STEP_LOWER_49:
        out     SREG,I_SREG
        reti
