; Shifter - serial input module for BLINKENmini, LittleLights and BlinkenLeds
; output code for LittleLights and BlinkenLeds
; version 0.1 date 2006-02-20
; Copyright (C) 2005-2006 Stefan Schuermans <1stein@schuermans.info>
; Copyleft: GNU public license - http://www.gnu.org/copyleft/gpl.html
; a BlinkenArea project - http://www.blinkenarea.org/



; output a black/white picture
;  - turn on all pixel with value >= 1
;  - uses: X, I_COL17, I_COL18, I_REG0, I_REG1, I_REG2, I_CNT, I_TMP
;  - used stack depth (without call to this function): 0
OUT_BW_1:
        mov     XH,OUT_PTR_OUT_E_H              ; begin at end of output buffer used for output
        mov     XL,OUT_PTR_OUT_E_L
        ld      I_REG2,-X                       ; get bits 18th column (Di of values in REGi)
        ld      I_REG1,-X
        ld      I_REG0,-X
        or      I_REG2,I_REG1                   ; >= 001: D2 | D1 | D0
        or      I_REG2,I_REG0
        mov     I_COL18,I_REG2
        ld      I_REG2,-X                       ; get bits 17th column (Di of values in REGi)
        ld      I_REG1,-X
        ld      I_REG0,-X
        or      I_REG2,I_REG1                   ; >= 001: D2 | D1 | D0
        or      I_REG2,I_REG0
        mov     I_COL17,I_REG2
        ldi     I_CNT,16                        ; 16 times 8 bit left
OUT_BW_1_LOOP:
        ld      I_REG2,-X                       ; get next bits (Di of values in REGi)
        ld      I_REG1,-X
        ld      I_REG0,-X
        or      I_REG2,I_REG1                   ; >= 001: D2 | D1 | D0
        or      I_REG2,I_REG0
        out     PORT_DATA,I_REG2                ; output data
        in      I_TMP,PORT_DATA9                ; output bit for 9th shift register chain
        cbr     I_TMP,1<<BIT_DATA9
        sbrc    I_COL18,7
        sbr     I_TMP,1<<BIT_DATA9
        out     PORT_DATA9,I_TMP
        lsl     I_COL17                         ; prepare next bit for 9th shift register chain (interleaved with clock output)
        sbi     PORT_CLK,BIT_CLK                ; output clock impulse(interleaved with bit preparation)
        rol     I_COL18
        cbi     PORT_CLK,BIT_CLK
        dec     I_CNT                           ; next turn
        brne    OUT_BW_1_LOOP
        sbi     PORT_STR,BIT_STR                ; output strobe impulse
        nop
        cbi     PORT_STR,BIT_STR
        ret

; output a black/white picture
;  - turn on all pixel with value >= 2
;  - uses: X, I_COL17, I_COL18, I_REG0, I_REG1, I_REG2, I_CNT, I_TMP
;  - used stack depth (without call to this function): 0
OUT_BW_2:
        mov     XH,OUT_PTR_OUT_E_H              ; begin at end of output buffer used for output
        mov     XL,OUT_PTR_OUT_E_L
        ld      I_REG2,-X                       ; get bits 18th column (Di of values in REGi)
        ld      I_REG1,-X
        ld      I_REG0,-X
        or      I_REG2,I_REG1                   ; >= 010: D2 | D1
        mov     I_COL18,I_REG2
        ld      I_REG2,-X                       ; get bits 17th column (Di of values in REGi)
        ld      I_REG1,-X
        ld      I_REG0,-X
        or      I_REG2,I_REG1                   ; >= 010: D2 | D1
        mov     I_COL17,I_REG2
        ldi     I_CNT,16                        ; 16 times 8 bit left
OUT_BW_2_LOOP:
        ld      I_REG2,-X                       ; get next bits (Di of values in REGi)
        ld      I_REG1,-X
        ld      I_REG0,-X
        or      I_REG2,I_REG1                   ; >= 010: D2 | D1
        out     PORT_DATA,I_REG2                ; output data
        in      I_TMP,PORT_DATA9                ; output bit for 9th shift register chain
        cbr     I_TMP,1<<BIT_DATA9
        sbrc    I_COL18,7
        sbr     I_TMP,1<<BIT_DATA9
        out     PORT_DATA9,I_TMP
        lsl     I_COL17                         ; prepare next bit for 9th shift register chain (interleaved with clock output)
        sbi     PORT_CLK,BIT_CLK                ; output clock impulse(interleaved with bit preparation)
        rol     I_COL18
        cbi     PORT_CLK,BIT_CLK
        dec     I_CNT                           ; next turn
        brne    OUT_BW_2_LOOP
        sbi     PORT_STR,BIT_STR                ; output strobe impulse
        nop
        cbi     PORT_STR,BIT_STR
        ret

; output a black/white picture
;  - turn on all pixel with value >= 3
;  - uses: X, I_COL17, I_COL18, I_REG0, I_REG1, I_REG2, I_CNT, I_TMP
;  - used stack depth (without call to this function): 0
OUT_BW_3:
        mov     XH,OUT_PTR_OUT_E_H              ; begin at end of output buffer used for output
        mov     XL,OUT_PTR_OUT_E_L
        ld      I_REG2,-X                       ; get bits 18th column (Di of values in REGi)
        ld      I_REG1,-X
        ld      I_REG0,-X
        and     I_REG1,I_REG0                   ; >= 011: D2 | (D1 & D0)
        or      I_REG2,I_REG1
        mov     I_COL18,I_REG2
        ld      I_REG2,-X                       ; get bits 17th column (Di of values in REGi)
        ld      I_REG1,-X
        ld      I_REG0,-X
        and     I_REG1,I_REG0                   ; >= 011: D2 | (D1 & D0)
        or      I_REG2,I_REG1
        mov     I_COL17,I_REG2
        ldi     I_CNT,16                        ; 16 times 8 bit left
OUT_BW_3_LOOP:
        ld      I_REG2,-X                       ; get next bits (Di of values in REGi)
        ld      I_REG1,-X
        ld      I_REG0,-X
        and     I_REG1,I_REG0                   ; >= 011: D2 | (D1 & D0)
        or      I_REG2,I_REG1
        out     PORT_DATA,I_REG2                ; output data
        in      I_TMP,PORT_DATA9                ; output bit for 9th shift register chain
        cbr     I_TMP,1<<BIT_DATA9
        sbrc    I_COL18,7
        sbr     I_TMP,1<<BIT_DATA9
        out     PORT_DATA9,I_TMP
        lsl     I_COL17                         ; prepare next bit for 9th shift register chain (interleaved with clock output)
        sbi     PORT_CLK,BIT_CLK                ; output clock impulse(interleaved with bit preparation)
        rol     I_COL18
        cbi     PORT_CLK,BIT_CLK
        dec     I_CNT                           ; next turn
        brne    OUT_BW_3_LOOP
        sbi     PORT_STR,BIT_STR                ; output strobe impulse
        nop
        cbi     PORT_STR,BIT_STR
        ret

; output a black/white picture
;  - turn on all pixel with value >= 4
;  - uses: X, I_COL17, I_COL18, I_REG0, I_REG1, I_REG2, I_CNT, I_TMP
;  - used stack depth (without call to this function): 0
OUT_BW_4:
        mov     XH,OUT_PTR_OUT_E_H              ; begin at end of output buffer used for output
        mov     XL,OUT_PTR_OUT_E_L
        ld      I_REG2,-X                       ; get bits 18th column (Di of values in REGi)
        ld      I_REG1,-X
        ld      I_REG0,-X
                                                ; >= 100: D2
        mov     I_COL18,I_REG2
        ld      I_REG2,-X                       ; get bits 17th column (Di of values in REGi)
        ld      I_REG1,-X
        ld      I_REG0,-X
                                                ; >= 100: D2
        mov     I_COL17,I_REG2
        ldi     I_CNT,16                        ; 16 times 8 bit left
OUT_BW_4_LOOP:
        ld      I_REG2,-X                       ; get next bits (Di of values in REGi)
        ld      I_REG1,-X
        ld      I_REG0,-X
                                                ; >= 100: D2
        out     PORT_DATA,I_REG2                ; output data
        in      I_TMP,PORT_DATA9                ; output bit for 9th shift register chain
        cbr     I_TMP,1<<BIT_DATA9
        sbrc    I_COL18,7
        sbr     I_TMP,1<<BIT_DATA9
        out     PORT_DATA9,I_TMP
        lsl     I_COL17                         ; prepare next bit for 9th shift register chain (interleaved with clock output)
        sbi     PORT_CLK,BIT_CLK                ; output clock impulse(interleaved with bit preparation)
        rol     I_COL18
        cbi     PORT_CLK,BIT_CLK
        dec     I_CNT                           ; next turn
        brne    OUT_BW_4_LOOP
        sbi     PORT_STR,BIT_STR                ; output strobe impulse
        nop
        cbi     PORT_STR,BIT_STR
        ret

; output a black/white picture
;  - turn on all pixel with value >= 5
;  - uses: X, I_COL17, I_COL18, I_REG0, I_REG1, I_REG2, I_CNT, I_TMP
;  - used stack depth (without call to this function): 0
OUT_BW_5:
        mov     XH,OUT_PTR_OUT_E_H              ; begin at end of output buffer used for output
        mov     XL,OUT_PTR_OUT_E_L
        ld      I_REG2,-X                       ; get bits 18th column (Di of values in REGi)
        ld      I_REG1,-X
        ld      I_REG0,-X
        or      I_REG1,I_REG0                   ; >= 101: D2 & (D1 | D0)
        and     I_REG2,I_REG1
        mov     I_COL18,I_REG2
        ld      I_REG2,-X                       ; get bits 17th column (Di of values in REGi)
        ld      I_REG1,-X
        ld      I_REG0,-X
        or      I_REG1,I_REG0                   ; >= 101: D2 & (D1 | D0)
        and     I_REG2,I_REG1
        mov     I_COL17,I_REG2
        ldi     I_CNT,16                        ; 16 times 8 bit left
OUT_BW_5_LOOP:
        ld      I_REG2,-X                       ; get next bits (Di of values in REGi)
        ld      I_REG1,-X
        ld      I_REG0,-X
        or      I_REG1,I_REG0                   ; >= 101: D2 & (D1 | D0)
        and     I_REG2,I_REG1
        out     PORT_DATA,I_REG2                ; output data
        in      I_TMP,PORT_DATA9                ; output bit for 9th shift register chain
        cbr     I_TMP,1<<BIT_DATA9
        sbrc    I_COL18,7
        sbr     I_TMP,1<<BIT_DATA9
        out     PORT_DATA9,I_TMP
        lsl     I_COL17                         ; prepare next bit for 9th shift register chain (interleaved with clock output)
        sbi     PORT_CLK,BIT_CLK                ; output clock impulse(interleaved with bit preparation)
        rol     I_COL18
        cbi     PORT_CLK,BIT_CLK
        dec     I_CNT                           ; next turn
        brne    OUT_BW_5_LOOP
        sbi     PORT_STR,BIT_STR                ; output strobe impulse
        nop
        cbi     PORT_STR,BIT_STR
        ret

; output a black/white picture
;  - turn on all pixel with value >= 6
;  - uses: X, I_COL17, I_COL18, I_REG0, I_REG1, I_REG2, I_CNT, I_TMP
;  - used stack depth (without call to this function): 0
OUT_BW_6:
        mov     XH,OUT_PTR_OUT_E_H              ; begin at end of output buffer used for output
        mov     XL,OUT_PTR_OUT_E_L
        ld      I_REG2,-X                       ; get bits 18th column (Di of values in REGi)
        ld      I_REG1,-X
        ld      I_REG0,-X
        and     I_REG2,I_REG1                   ; >= 110: D2 & D1
        mov     I_COL18,I_REG2
        ld      I_REG2,-X                       ; get bits 17th column (Di of values in REGi)
        ld      I_REG1,-X
        ld      I_REG0,-X
        and     I_REG2,I_REG1                   ; >= 110: D2 & D1
        mov     I_COL17,I_REG2
        ldi     I_CNT,16                        ; 16 times 8 bit left
OUT_BW_6_LOOP:
        ld      I_REG2,-X                       ; get next bits (Di of values in REGi)
        ld      I_REG1,-X
        ld      I_REG0,-X
        and     I_REG2,I_REG1                   ; >= 110: D2 & D1
        out     PORT_DATA,I_REG2                ; output data
        in      I_TMP,PORT_DATA9                ; output bit for 9th shift register chain
        cbr     I_TMP,1<<BIT_DATA9
        sbrc    I_COL18,7
        sbr     I_TMP,1<<BIT_DATA9
        out     PORT_DATA9,I_TMP
        lsl     I_COL17                         ; prepare next bit for 9th shift register chain (interleaved with clock output)
        sbi     PORT_CLK,BIT_CLK                ; output clock impulse(interleaved with bit preparation)
        rol     I_COL18
        cbi     PORT_CLK,BIT_CLK
        dec     I_CNT                           ; next turn
        brne    OUT_BW_6_LOOP
        sbi     PORT_STR,BIT_STR                ; output strobe impulse
        nop
        cbi     PORT_STR,BIT_STR
        ret

; output a black/white picture
;  - turn on all pixel with value >= 7
;  - uses: X, I_COL17, I_COL18, I_REG0, I_REG1, I_REG2, I_CNT, I_TMP
;  - used stack depth (without call to this function): 0
OUT_BW_7:
        mov     XH,OUT_PTR_OUT_E_H              ; begin at end of output buffer used for output
        mov     XL,OUT_PTR_OUT_E_L
        ld      I_REG2,-X                       ; get bits 18th column (Di of values in REGi)
        ld      I_REG1,-X
        ld      I_REG0,-X
        and     I_REG2,I_REG1                   ; >= 111: D2 & D1 & D0
        and     I_REG2,I_REG0
        mov     I_COL18,I_REG2
        ld      I_REG2,-X                       ; get bits 17th column (Di of values in REGi)
        ld      I_REG1,-X
        ld      I_REG0,-X
        and     I_REG2,I_REG1                   ; >= 111: D2 & D1 & D0
        and     I_REG2,I_REG0
        mov     I_COL17,I_REG2
        ldi     I_CNT,16                        ; 16 times 8 bit left
OUT_BW_7_LOOP:
        ld      I_REG2,-X                       ; get next bits (Di of values in REGi)
        ld      I_REG1,-X
        ld      I_REG0,-X
        and     I_REG2,I_REG1                   ; >= 111: D2 & D1 & D0
        and     I_REG2,I_REG0
        out     PORT_DATA,I_REG2                ; output data
        in      I_TMP,PORT_DATA9                ; output bit for 9th shift register chain
        cbr     I_TMP,1<<BIT_DATA9
        sbrc    I_COL18,7
        sbr     I_TMP,1<<BIT_DATA9
        out     PORT_DATA9,I_TMP
        lsl     I_COL17                         ; prepare next bit for 9th shift register chain (interleaved with clock output)
        sbi     PORT_CLK,BIT_CLK                ; output clock impulse(interleaved with bit preparation)
        rol     I_COL18
        cbi     PORT_CLK,BIT_CLK
        dec     I_CNT                           ; next turn
        brne    OUT_BW_7_LOOP
        sbi     PORT_STR,BIT_STR                ; output strobe impulse
        nop
        cbi     PORT_STR,BIT_STR
        ret
