; Shifter - serial input module for BLINKENmini, LittleLights and BlinkenLeds
; reception code
; version 0.1 date 2006-02-20
; Copyright (C) 2005-2006 Stefan Schuermans <1stein@schuermans.info>
; Copyleft: GNU public license - http://www.gnu.org/copyleft/gpl.html
; a BlinkenArea project - http://www.blinkenarea.org/



; serial reception
; uses: Z, CNT, TMP, REG0, REG1, REG2
;  - used stack depth (without call to this function): 1
RECV:
; check for reception
        sbis    UCSRA,RXC
        rjmp    RECV_END                        ; nothing received
; check for errors
        sbic    UCSRA,FE
        rjmp    RECV_ERR                        ; frame error
        sbic    UCSRA,DOR
        rjmp    RECV_ERR                        ; data overrun
; check if behind message
        cpi     RECV_STEP,12+18*8
        brlo    RECV_STEP_OK
        clr     RECV_STEP                       ; behind message -> start receiving a new message
RECV_STEP_OK:        
; check if in header or in data part
        cpi     RECV_STEP,12
        brlo    RECV_HEADER
        rjmp    RECV_DATA

; receive error
RECV_ERR:
        in      TMP,UDR                         ; read data from USART to clear error
        clr     RECV_STEP                       ; start receiving a new message
RECV_END:
        ret

; expected header data (MCUF header 18x8-1/256)
RECV_HEADER_DATA:
        .db     0x23, 0x54, 0x26, 0x66          ; magic (first byte is also contained in code)
        .db     0x00, 0x08                      ; height: 8
        .db     0x00, 0x12                      ; width: 18
        .db     0x00, 0x01                      ; channels: 1
        .db     0x00, 0xFF                      ; maxval: 255

; received a header byte
RECV_HEADER:
        in      REG0,UDR                        ; fetch data byte from USART
        ldi     ZH,high(2*RECV_HEADER_DATA)     ; load base address of header data ("2*" to convert word address to byte address)
        ldi     ZL,low(2*RECV_HEADER_DATA)
        clr     TMP                             ; add offset
        add     ZL,RECV_STEP
        adc     ZH,TMP
        lpm     REG1,Z                          ; fetch expected data byte from program memory
        cp      REG0,REG1                       ; compare with received data byte
        breq    RECV_HEADER_OK
        clr     RECV_STEP                       ; mismatch -> start receiving new message
        ldi     TMP,0x23                        ; check if first byte of message (first byte of magic)
        cp      REG0,TMP
        breq    RECV_HEADER_OK                  ; first byte of message/magic -> accept it
        ret
RECV_HEADER_OK:
        inc     RECV_STEP                       ; match -> next byte
        clr     RECV_X                          ; next pixel to receive is topleft pixel
        clr     RECV_Y                          ;  - only needed for last header byte, but faster and more easy this way
        ret

; received a data byte
RECV_DATA:
        in      REG0,UDR                        ; fetch data byte from USART
        rcall   RECV_STORE                      ; store received byte in receive buffer
        inc     RECV_STEP                       ; next byte
        inc     RECV_X                          ; next pixel
        ldi     TMP,18
        cp      RECV_X,TMP
        brlo    RECV_NEXT_END
        clr     RECV_X                          ; next line
        inc     RECV_Y
RECV_NEXT_END:
        cpi     RECV_STEP,12+18*8               ; end of frame - output picture
        brsh    RECV_DATA_OUTPUT
        ret

; output picture
RECV_DATA_OUTPUT:
; reset timeout timer
        cli                                     ; turn off interrupts
        clr     TMP                             ; reset timer 1 (timeout timer)
        out     TCNT1H,TMP
        out     TCNT1L,TMP
        in      TMP,OCF1A                       ; clear timeout flag (timer 1 compare match)
        sbr     TMP,1<<OCF1A
        out     TIFR,TMP
        sei                                     ; turn on interrupts
; show received data
        rjmp    OUT_FLIP                        ; show received data
